#include "grid.h"
#include <stdio.h>
#include <stdlib.h>

//

int coordonner_vers_indice(struct Grid *grid, int col, int row)
{
  return ((row * grid->column_number) + col);
}

enum CaseType *init_grid(int width, int height)
{
  size_t taille_grid = width * height;
  enum CaseType *new_grid = (enum CaseType *)malloc(taille_grid * sizeof(enum CaseType));
  for (size_t i = 0; i < taille_grid; i++)
  {
    new_grid[i] = NONE;
  }
  return new_grid;
}

struct Player *init_player()
{
  struct Player *player = (struct Player *)malloc(sizeof(struct Player));
  player->col = 0;
  player->row = 0;
  return player;
}

struct Grid *new_grid(int width, int height)
{
  struct Grid *new_strut_grid = (struct Grid *)malloc(sizeof(struct Grid));
  new_strut_grid->column_number = width;
  new_strut_grid->row_number = height;
  new_strut_grid->game_grid = init_grid(width, height);
  new_strut_grid->position = init_player();
  new_strut_grid->tabGoal = NULL;
  return new_strut_grid;
}

struct Grid *init_level(const char *file_path)
{
  // ouverture du fichier en mode lecture
  FILE *file = fopen(file_path, "r");
  if (!file)
  {
    fprintf(stderr, "Error %s not found", file_path);
    exit(-1);
  }
  char line[100] = {0};
  int number_column = 0; // nombre de colonne
  int number_row = 0;    /// nombre de ligne
  int number_goals = 0;
  // on lit la première ligne du fichier
  fgets(line, 100, file);
  sscanf(line, "%d %d %d", &number_column, &number_row, &number_goals);
  struct Grid *grid = new_grid(number_column, number_row);
  int current_row = 0;
  grid->number_goal = number_goals;
  grid->tabGoal = malloc(number_goals * (sizeof(struct Goal)));
  int current_goal = 0;
  // On lit le fichier ligne par ligne jusqu'à la fin du fichier
  while (fgets(line, 100, file) != NULL)
  {
    char *buffer = line;
    int current_column = 0;
    while (*buffer && *buffer != '\n')
    {
      if (buffer[0] == '@')
      {
        grid->position->col = current_column;
        grid->position->row = current_row;
      }
      else if (buffer[0] == '.')
      {
        grid->tabGoal[current_goal].col = current_column;
        grid->tabGoal[current_goal].row = current_row;
        current_goal = +1;
      }
      else
      {
        change_cell_grid(grid, current_column, current_row, buffer[0]);
      }
      current_column += 1;
      buffer += 1;
    }
    current_row += 1;
  }

  // fermeture du fichier
  fclose(file);
  return grid;
}

enum CaseType get_grid(struct Grid *grid, int col, int row)
{
  if (0 <= col && col < grid->column_number && 0 <= row && row < grid->row_number)
  {
    return grid->game_grid[coordonner_vers_indice(grid, col, row)];
  }
  else
  {
    return -1;
  }
}

int get_goal(struct Grid *grid, int col, int row)
{
  if (0 <= col && col < grid->column_number && 0 <= row && row < grid->row_number)
  {
    for (int indice = 0; indice < grid->number_goal; indice++)
    {
      if (grid->tabGoal[indice].col == col && grid->tabGoal[indice].row == row)
      {
        return 1;
      }
    }
    return -1;
  }
  else
  {
    return -1;
  }
}

void Affichage_grid(struct Grid *grid)
{
  for (int row = 0; row < grid->row_number; row++)
  {
    for (int col = 0; col < grid->column_number; col++)
    {
      switch (get_grid(grid, col, row))
      {
      case NONE: {
        if (grid->position->col == col && grid->position->row == row)
        {
          printf("@");
        }
        else if (get_goal(grid, col, row) == 1)
        {
          printf(".");
        }
        else
        {
          printf(" ");
        }
        break;
      }
      case WALL: {
        printf("#");
        break;
      }
      case BOX: {
        printf("$");
        break;
      }
      case PLAYER: {
        printf("@");
        break;
      }
      case GOAL: {
        printf(".");
        break;
      }
      }
    }
    printf("\n");
  }
}

void change_cell_grid(struct Grid *grid, int col, int row, enum CaseType new_valeur)
{
  grid->game_grid[coordonner_vers_indice(grid, col, row)] = new_valeur;
}

void free_grid(struct Grid *grid)
{
  free(grid->game_grid);
  free(grid->tabGoal);
  free(grid->position);
  free(grid);
}
